@@ -17,8 +17,6 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-/* $Id: Resolver.cc,v 1.90 2011/03/25 14:35:43 lav Exp $ */
-
 #include <config.h>
 
 #include <stddef.h>
@@ -212,7 +210,7 @@ int   Resolver::Do()
 	 LogNote(4,_("Resolving host address..."));
 	 buf=new IOBuffer(IOBuffer::GET);
 	 DoGethostbyname();
-	 if(deleting)
+	 if(Deleted())
 	    return MOVED;
       }
    }
@@ -492,7 +490,7 @@ void Resolver::LookupSRV_RR()
       if(!use_fork)
       {
 	 Schedule();
-	 if(deleting)
+	 if(Deleted())
 	    return;
       }
       time(&try_time);
@@ -690,7 +688,7 @@ void Resolver::LookupOne(const char *name)
       if(!use_fork)
       {
 	 Schedule();
-	 if(deleting)
+	 if(Deleted())
 	    return;
       }
 
@@ -726,11 +724,17 @@ void Resolver::LookupOne(const char *name)
       bool require_trust=ResMgr::QueryBool("dns:strict-dnssec",name);
       ainfo_res	= val_getaddrinfo(NULL, name, NULL, &a_hint, &ainfo,
                                   &val_status);
-      if(VAL_GETADDRINFO_HAS_STATUS(ainfo_res)
-      && !val_istrusted(val_status) && require_trust) {
-          // untrusted answer
-          error = _("DNS resoloution not trusted.");
-          break;
+      if(VAL_GETADDRINFO_HAS_STATUS(ainfo_res) && !val_istrusted(val_status))
+      {
+          if(require_trust) {
+              // untrusted answer
+              error = _("DNS resolution not trusted.");
+              break;
+          } else {
+              fprintf(stderr,"\nWARNING: DNS lookup failed validation: %s\n",
+                      p_val_status(val_status));
+	      fflush(stderr);
+          }
       }
 #endif
 
@@ -871,12 +875,12 @@ void Resolver::DoGethostbyname()
    if(service && !portname && !isdigit((unsigned char)hostname[0]))
       LookupSRV_RR();
 
-   if(!use_fork && deleting)
+   if(!use_fork && Deleted())
       return;
 
    LookupOne(hostname);
 
-   if(!use_fork && deleting)
+   if(!use_fork && Deleted())
       return;
 
    if(addr.count()==0)
