@@ -219,11 +219,16 @@ class Torrent : public SMTask, protected ProtoLog, public ResClient
    static int PeersCompareRecvRate(const SMTaskRef<TorrentPeer> *p1,const SMTaskRef<TorrentPeer> *p2);
    static int PeersCompareSendRate(const SMTaskRef<TorrentPeer> *p1,const SMTaskRef<TorrentPeer> *p2);
 
+   void RebuildPiecesNeeded();
    Timer pieces_needed_rebuild_timer;
    xarray<unsigned> pieces_needed;
    static int PiecesNeededCmp(const unsigned *a,const unsigned *b);
    unsigned last_piece;
 
+   unsigned min_piece_sources;
+   unsigned avg_piece_sources;
+   unsigned pieces_available_pct;
+
    void SetPieceNotWanted(unsigned piece);
    void SetDownloader(unsigned piece,unsigned block,const TorrentPeer *o,const TorrentPeer *n);
 
@@ -300,7 +305,7 @@ public:
    const Error *GetInvalidCause() const { return invalid_cause; }
 
    void Shutdown();
-   bool ShuttingDown() { return shutting_down; }
+   bool ShuttingDown() const { return shutting_down; }
    void PrepareToDie();
 
    bool CanAccept() const;
@@ -325,9 +330,14 @@ public:
    bool Complete() const { return complete; }
    bool Private() const { return is_private; }
    double GetRatio() const;
+   void CalcPiecesStats();
+   unsigned MinPieceSources() const { return min_piece_sources; }
+   double AvgPieceSources() const { return avg_piece_sources/256.; }
+   unsigned PiecesAvailablePct() const { return pieces_available_pct; }
    unsigned long long TotalLength() const { return total_length; }
    unsigned PieceLength() const { return piece_length; }
    const char *GetName() const { return name?name.get():metainfo_url.get(); }
+   bool IsDownloading() const { return HasMetadata() && !IsValidating() && !Complete() && !ShuttingDown(); }
 
    void Reconfig(const char *name);
    const char *GetLogContext() { return GetName(); }
@@ -351,7 +361,7 @@ public:
    unsigned long long GetTotalLeft() { return total_left; }
 
    const TaskRefArray<TorrentTracker>& Trackers() { return trackers; }
-   bool HasMetadata() { return metadata!=0; }
+   bool HasMetadata() const { return metadata!=0; }
    void DisconnectPeers();
 
    static void BootstrapDHT(const char *n) {
